import 'package:niralanlayout/utils/settings_persistence_utils.dart';
import 'package:flutter/foundation.dart';

class SettingsController {
  final SettingsPersistenceUtils _persistence;

  ValueNotifier<bool> muted = ValueNotifier(false);

  ValueNotifier<String> type = ValueNotifier("4x4");

  ValueNotifier<bool> reviewStatus = ValueNotifier(false);

  ValueNotifier<int> level = ValueNotifier(1);

  ValueNotifier<bool> soundsOn = ValueNotifier(false);

  ValueNotifier<bool> musicOn = ValueNotifier(false);

  SettingsController({required SettingsPersistenceUtils persistence})
      : _persistence = persistence;

  Future<void> loadStateFromPersistence() async {
    await Future.wait(
      [
        _persistence
            .getMuted(defaultValue: kIsWeb)
            .then((value) => muted.value = value),
        _persistence.getSoundsOn().then((value) => soundsOn.value = value),
        _persistence.getMusicOn().then((value) => musicOn.value = value),
        _persistence
            .getReviewStatus()
            .then((value) => reviewStatus.value = value),
        _persistence.getLevel().then((value) {
          level.value = value;
        }),
        _persistence.getType().then((value) {
          type.value = value;
        }),
      ],
    );
  }

  void setReview(bool name) {
    reviewStatus.value = name;
    _persistence.saveReviewStatus(reviewStatus.value);
  }

  void setLevel(int input) {
    level.value = input;
    _persistence.saveLevel(level.value);
  }

  void setType(String input) {
    type.value = input;
    _persistence.saveType(type.value);
  }

  void toggleMusicOn() {
    musicOn.value = !musicOn.value;
    _persistence.saveMusicOn(musicOn.value);
  }

  void toggleMuted() {
    muted.value = !muted.value;
    _persistence.saveMuted(muted.value);
  }

  void toggleSoundsOn() {
    soundsOn.value = !soundsOn.value;
    _persistence.saveSoundsOn(soundsOn.value);
  }
}
