import 'package:flutter/material.dart';
import 'package:niralanlayout/widgets/custom_tile_widget.dart';

class CustomAlertDialog extends StatelessWidget {
  final String title;
  final String message;
  final String background;
  final String? positiveButtonText;
  final String? negativeButtonText;
  final Function? onPositivePressed;
  final Function? onNegativePressed;
  final double circularBorderRadius;
  final double buttonWidth;
  final double buttonHeight;
  final String fontFamily;
  final Color fontColor;
  final double fontSize;
  final Color secondaryFontColor;
  final String titleBox;

  CustomAlertDialog({
    required this.title,
    required this.message,
    required this.background,
    this.circularBorderRadius = 15.0,
    this.positiveButtonText,
    this.negativeButtonText,
    this.onPositivePressed,
    this.onNegativePressed,
    required this.fontFamily,
    required this.fontColor,
    required this.fontSize,
    required this.secondaryFontColor,
    required this.titleBox,
    required this.buttonWidth,
    required this.buttonHeight,
  });

  @override
  Widget build(BuildContext context) {
    return AlertDialog(
      contentPadding: EdgeInsets.all(2),
      content: Container(
        height: 300,
        decoration: BoxDecoration(
          color: Colors.transparent,
          image: DecorationImage(
            image: AssetImage(background),
            fit: BoxFit.fill,
          ),
        ),
        child: Padding(
          padding: EdgeInsets.all(15),
          child: Column(
            mainAxisAlignment: MainAxisAlignment.start,
            children: <Widget>[
              Container(
                child: Text(
                  title,
                  style: TextStyle(
                    fontFamily: fontFamily,
                    fontSize: fontSize,
                    color: secondaryFontColor,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
              Container(
                child: Text(
                  message,
                  style: TextStyle(
                    fontFamily: fontFamily,
                    fontSize: 18,
                    color: secondaryFontColor,
                    fontWeight: FontWeight.bold,
                  ),
                ),
              ),
              Visibility(
                visible: onPositivePressed != null,
                child: CustomTileWidget(
                  fontFamily: fontFamily,
                  buttonHeight: buttonHeight,
                  buttonWidth: buttonWidth,
                  onTap: () {
                    onPositivePressed!();
                  },
                  textColor: fontColor,
                  text: positiveButtonText!,
                  fontSize: 18,
                  image: titleBox,
                ),
              ),
              Visibility(
                visible: onNegativePressed != null,
                child: CustomTileWidget(
                  fontFamily: fontFamily,
                  buttonHeight: buttonHeight,
                  buttonWidth: buttonWidth,
                  onTap: () {
                    onNegativePressed!();
                  },
                  textColor: fontColor,
                  text: negativeButtonText!,
                  fontSize: 28,
                  image: titleBox,
                ),
              ),
            ],
          ),
        ),
      ),
    );
  }
}
